library jaguar;

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'dart:convert';
import 'dart:io';
import 'package:http/http.dart' show Client;
import 'package:flutter/foundation.dart' show kIsWeb;

class BotonRest extends StatefulWidget {
  const BotonRest(
      {Key key,
      @required this.contenido,
      this.estilo,
      @required this.url,
      @required this.method,
      this.action,
      this.formkey,
      this.arrayData,
      this.scaffoldkey,
      this.errorStyle,
      @required this.urlApi})
      : super(key: key);

  final Widget contenido;
  final BoxDecoration estilo;
  final String url;
  final String method;
  final action;
  final formkey;
  final dynamic arrayData;
  final scaffoldkey;
  final TextStyle errorStyle;
  final String urlApi;

  @override
  _BotonRestState createState() => _BotonRestState();
}

class _BotonRestState extends State<BotonRest> {
  bool load = false;
  bool error = false;
  String errorString = 'Algo salio mal';

  Rest restFunction = Rest();

  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        error
            ? Padding(
                padding: const EdgeInsets.only(bottom: 15),
                child: Text(
                  errorString,
                  textAlign: TextAlign.center,
                  style: widget.errorStyle,
                ),
              )
            : Container(),
        Container(
          decoration: widget.estilo,
          child: Material(
            clipBehavior: Clip.hardEdge,
            color: Colors.transparent,
            child: InkWell(
              highlightColor: Colors.grey.withOpacity(0.2),
              onTap: () {
                if (!load) {
                  if (widget.formkey.currentState.validate()) {
                    widget.formkey.currentState.save();
                    setState(() {
                      load = true;
                    });
                    restFunction
                        .restService(widget.urlApi, widget.arrayData,
                            widget.url, null, widget.method)
                        .then((serverResp) {
                      if (serverResp['status'] == 'server_true') {
                        widget.action;
                        setState(() {
                          load = true;
                        });
                      } else {
                        setState(() {
                          load = false;
                          error = true;
                          errorString = serverResp['message'].toString();
                        });
                      }
                    });
                  }
                }
              },
              child: Container(
                  padding: EdgeInsets.symmetric(vertical: 10, horizontal: 5),
                  alignment: Alignment.center,
                  child: load
                      ? Container(
                          height: 18,
                          width: 18,
                          child: CircularProgressIndicator(
                            backgroundColor: Colors.white,
                            strokeWidth: 3,
                          ))
                      : widget.contenido),
            ),
          ),
        )
      ],
    );
  }
}

class Rest {
  Client client = Client();
  BuildContext context;

  Future<dynamic> restService(String urlApi, dynamic arrayData, String ruta,
      String token, String method) async {
    if (!kIsWeb) {
      try {
        final result = await InternetAddress.lookup('google.com');
        if (result.isNotEmpty && result[0].rawAddress.isNotEmpty) {
          //Connected
        }
      } on SocketException catch (_) {
        // Not connected
        if (!Get.isSnackbarOpen)
          Get.snackbar(
            'Ha ocurrido un error', // title
            'Parece que no tienes conexión a internet', // message
            colorText: Colors.white,
            backgroundColor: Colors.red[600],
            barBlur: 20,
            isDismissible: true,
            snackPosition: SnackPosition.TOP,
            duration: Duration(seconds: 5),
            animationDuration: Duration(milliseconds: 300),
          );
      }
    }

    var response;
    String requestStatus = "local_false";
    String requestResponse = "Hubo un error local para procesar la petición";
    String messageToUser;

    switch (method) {
      case 'post':
        response = await client.post(Uri.parse('$urlApi$ruta'),
            headers: {
              'Content-Type': 'application/json',
              'Authorization': 'Bearer $token',
            },
            body: jsonEncode(arrayData));
        break;
      case 'put':
        response = await client.put(Uri.parse('$urlApi$ruta'),
            headers: {
              'Content-Type': 'application/json',
              'Authorization': 'Bearer $token',
            },
            body: jsonEncode(arrayData));
        break;
      case 'delete':
        response = await client.delete(
          Uri.parse('$urlApi$ruta'),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer $token',
          },
        );
        break;
      case 'get':
        response = await client.get(
          Uri.parse('$urlApi$ruta'),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer $token',
          },
        );
        break;
      default:
        response = await client.get(
          Uri.parse('$urlApi$ruta'),
          headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer $token',
          },
        );
        break;
    }
    var jsonResponse = jsonDecode(response.body);
    if (response.statusCode == 200) {
      if (jsonResponse.length == 4) {
        if (jsonResponse['status'] == "true") {
          requestStatus = "server_true";
          requestResponse = jsonEncode(jsonResponse);
          print("Todo bien:" + jsonResponse.toString());
        } else {
          requestStatus = "server_false";
          requestResponse = jsonEncode(jsonResponse);
          messageToUser = jsonResponse['message'].toString();
          print("Hay un problema: " + jsonResponse['message'].toString());
        }
      } else {
        if (jsonResponse[0]['status'] == "true") {
          requestStatus = "server_true";
          requestResponse = jsonEncode(jsonResponse);
          print("Todo bien:" + jsonResponse.toString());
        } else {
          requestStatus = "server_false";
          requestResponse = jsonEncode(jsonResponse);
          messageToUser = jsonResponse[0]['message'].toString();
          print("Hay un problema: " + jsonResponse[0]['message'].toString());
        }
      }
    } else {
      requestStatus = "server_false";
      requestResponse = jsonEncode(jsonResponse);
      print("Hubo un problema para realizar la petición");
      print(response.body);
    }

    return {
      "status": requestStatus,
      "response": requestResponse,
      "message": messageToUser.toString()
    };
  }
}
